﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.Audio;
using UnityEngine.SceneManagement;
using UnityEngine.UI;

public class Menu : MonoBehaviour {

    public AudioMixer MasterMixer;
    public AudioMixer MusicMixer;
    public AudioMixer EffectMixer;

    public Dropdown ResolutionDropdown;
    private Resolution[] resolutions;
    
    private void Start() {
        resolutions = Screen.resolutions;
        ResolutionDropdown.ClearOptions();
        
        var options = new List<string>();

        int currentResolutionIndex = 0;
        
        for (int i = 0; i < resolutions.Length; i++) {
            string option = resolutions[i].width + " x " + resolutions[i].height;
            options.Add(option);

            if (resolutions[i].width == Screen.currentResolution.width 
                && resolutions[i].height == Screen.currentResolution.height) {
                currentResolutionIndex = i; 
            }
        }

        ResolutionDropdown.AddOptions(options);
        ResolutionDropdown.value = currentResolutionIndex;
        ResolutionDropdown.RefreshShownValue();
        
    }

    public void PlayButtonPressed() {
        SceneManager.LoadScene(1);
    }
    
    public void QuitButtonPressed() {
        Application.Quit();
    }

    public void setMasterVolume(float volume) {
        MasterMixer.SetFloat("MasterVolume", volume);
    }

    public void setMusicVolume(float volume) {
        MusicMixer.SetFloat("MusicVolume", volume);
    }

    public void setEffectVolume(float volume) {
        EffectMixer.SetFloat("EffectVolume", volume);
    }

    public void setQuality(int qualityIndex) {
        QualitySettings.SetQualityLevel(qualityIndex);
    }

    public void setFullscreen(bool isFullscreen) {
        Screen.fullScreen = isFullscreen;
    }
    
    public void setResolution(int resolutionIndex) {
        Resolution res = resolutions[resolutionIndex];
        Screen.SetResolution(res.width, res.height, Screen.fullScreen); 
    }
}