﻿using System.Collections;
using System.Collections.Generic;
//using UnityEditor;
using UnityEngine;

public class VaiselController : MonoBehaviour
{
	public float timeOnPlanet;
	Vaisel vaisel;
	Vector2 targetPosition;
	Vector3 orientation;
	int index;
	int speed;
	bool isOnPlanet;
	List<Vector2> allPoints;
	float startTime;

	bool reverse;

	private bool _shouldLoad;
	private PlanetDisplay _planet;
	private Dictionary<string, float> _soute;

	// Use this for initialization
	void Awake()
	{
        allPoints = new List<Vector2>();
		reverse = false;
		speed = 1;
		index = 0;
		vaisel = GetComponent<VaiselDisplay>().Vaisel;
		vaisel.Point = Vector2.zero;
		isOnPlanet = false;
		_soute = new Dictionary<string, float>(3);
		_soute["metal"] = 0;
		_soute["quartz"] = 0;
		_soute["voidium"] = 0;
		_shouldLoad = true;
	}

	// Update is called once per frame
	void Update()
	{
		MoveVaissel();
	}

	public Dictionary<string, float> Soute
	{
		get { return _soute; }
	}

    public void SetPath(List<Path> paths)
	{
        if(allPoints.Count > 0 && allPoints[0] != paths[0].GetStartPoint())
        {
            index += paths[0].GetPoints().Length - 1;
        }
		allPoints = new List<Vector2>();
		foreach (Path p in paths)
		{
			for (int i = 0; i < p.GetPoints().Length - 1; i++)
			{
				allPoints.Add(p.GetPoints()[i]);
			}
		}
		allPoints.Add(paths[paths.Count - 1].GetEndPoint());
	}

	private void MoveVaiselTo()
	{
		transform.position = Vector2.MoveTowards(transform.position, targetPosition, speed * Time.deltaTime);
	}

	private void OnTriggerEnter2D(Collider2D collision)
	{
		Debug.Log(collision.gameObject.tag);
		if (collision.gameObject.CompareTag("Planet"))
		{
			Planet = collision.gameObject.GetComponent<PlanetDisplay>();
			isOnPlanet = true;
			startTime = Time.time;
		}
	}

	private void MoveVaissel()
	{
		if (allPoints != null && allPoints.Count > 0)
		{
			if (vaisel.Point == Vector2.zero)
			{
				if (!reverse)
				{
					index = 0;
					transform.position = allPoints[index];
					vaisel.Point = allPoints[index];
					index++;
					targetPosition = allPoints[index];
                    UpdateRotation();
				}
				else
				{
					index = allPoints.Count - 1;
					transform.position = allPoints[index];
					vaisel.Point = allPoints[index];
					index--;
					targetPosition = allPoints[index];
                    UpdateRotation();
                }
			}
			if (new Vector2(transform.position.x, transform.position.y) == targetPosition)
			{
				vaisel.Point = targetPosition;
				if (!reverse)
					index++;
				else
					index--;
				if (index < 0 || index > allPoints.Count - 1)
				{
					reverse = !reverse;
					vaisel.Point = Vector2.zero;
				}
				else
				{
					targetPosition = allPoints[index];
                    UpdateRotation();
                }
			}
			if (!isOnPlanet)
			{
				MoveVaiselTo();
			}
			else
			{
				if (Time.time - startTime > timeOnPlanet)
				{
					isOnPlanet = false;
					ShouldLoad = true;
				}
			}
		}
	}

	public float Metal
	{
		get { return _soute["metal"]; }
		set { _soute["metal"] = value; }
	}

	public float Quartz
	{
		get { return _soute["quartz"]; }
		set { _soute["quartz"] = value; }
	}

	public float Voidium
	{
		get { return _soute["voidium"]; }
		set { _soute["voidium"] = value; }
	}

	public bool ShouldLoad
	{
		get { return _shouldLoad; }
		set { _shouldLoad = value; }
	}

	public PlanetDisplay Planet
	{
		get { return _planet; }
		set { _planet = value; }
	}

	public bool IsOnPlanet
	{
		get { return isOnPlanet; }
		set { isOnPlanet = value; }
	}

	/**
	 * Chargement des ressources
	 */
	public void Load()
	{
		// On récupère les trois ressources.
		Metal += Planet.LoadMetal(vaisel.MetalMax - _soute["metal"]);
		Quartz += _planet.LoadQuartz(vaisel.QuartzMax - _soute["quartz"]);
		Voidium += _planet.LoadVoidium(vaisel.VoidiumMax - _soute["voidium"]);
		_shouldLoad = false;
	}

	public void Print()
	{
		Debug.Log(string.Format("Metal : {0}, Quartz: {1}, Voidium: {2}", arg0: Metal, arg1: Quartz, arg2: Voidium));
	}
	
	
	private void UpdateRotation()
	{
		Vector3 orientation = new Vector3(targetPosition.x - transform.position.x, targetPosition.y - transform.position.y,0);
		orientation.Normalize();
		float rot_z = Mathf.Atan2(orientation.y, orientation.x) * Mathf.Rad2Deg;
		transform.rotation = Quaternion.Euler(0f, 0f, rot_z - 90);
	}
}